#!/usr/bin/env python3
"automig_sqlite.py -- entrypoint for sqlite migrations"

import sqlite3
from .migrate_pg import create_parser, init, update

def main():
  args = create_parser(__doc__).parse_args()
  assert args.automig_con, "pass --automig-con param or AUTOMIG_CON in env"
  if args.command == 'update':
    update(args, connect=sqlite3.connect, dialect='sqlite')
  elif args.command == 'init':
    init(args, connect=sqlite3.connect, dialect='sqlite')
  else:
    raise ValueError('unknown command', args.command)

if __name__ == '__main__': main()



#!/usr/bin/env python3
"migrate_pg.py -- . requires psycopg2, i.e. you should install with 'postgres' optional requirement"

import argparse, os
import psycopg2 # pylint: disable=import-error
from .__main__ import build_parser, main_inner

def init(args, connect=psycopg2.connect, dialect='postgres'):
  "body for `init` command"
	# eval "automig $TARGET $AUTOMIG_GLOB --initial" | psql $AUTOMIG_CON --single-transaction
  sql = main_inner(build_parser().parse_args([args.target, args.glob, '--initial', '--dialect', dialect]))
  print(sql)
  if args.preview:
    return
  con = connect(args.automig_con)
  if dialect == 'sqlite':
    cur = con.cursor()
    cur.execute('begin')
    cur.executescript(sql)
  else:
    con.cursor().execute(sql)
  con.commit()

def update(args, connect=psycopg2.connect, dialect='postgres'):
  "body for `update` command"
  con = connect(args.automig_con)
  cur = con.cursor()
  cur.execute('begin')
  cur.execute('select sha from automigrate_meta order by id desc limit 1')
  last_sha, = cur.fetchone()
  range_ = f"{last_sha}...{args.target}"
  print("range is", range_)
  pass_down_args = [range_, args.glob, '--dialect', dialect]
  if args.opaque:
    pass_down_args.append('--opaque')
  sql = main_inner(build_parser().parse_args(pass_down_args))
  print(sql)
  if args.preview:
    return
  empty = not any(not line.startswith('--') for line in sql.splitlines())
  if empty:
    print('update is empty, skipping')
  else:
    if dialect == 'sqlite':
      cur.executescript(sql)
    else:
      cur.execute(sql)
    con.commit()

def create_parser(doc):
  parser = argparse.ArgumentParser(description=doc)
  parser.add_argument('--target', default="HEAD", help="Target sha or git ref to migrate to / initialize with")
  parser.add_argument('--glob', default="schema/*.sql", help="Path glob, gets passed down to automig")
  parser.add_argument('--preview', action="store_true", help="Do a dry-run. Show what the change would be without applying it")
  parser.add_argument('--automig-con', default=os.environ.get('AUTOMIG_CON'), help="DB connection string. if missing, tries to get AUTOMIG_CON from env")
  sub = parser.add_subparsers(dest='command')
  sub.add_parser('init', help="initialize a schema at target, install meta tables if missing")
  cmd_update = sub.add_parser('update', help="Read latest commit from DB and update to target")
  cmd_update.add_argument('--opaque', action='store_true', help="Pass down to automig")
  return parser

def main():
  args = create_parser(__doc__).parse_args()
  assert args.automig_con, "pass --automig-con param or AUTOMIG_CON in env"
  if args.command == 'update':
    update(args)
  elif args.command == 'init':
    init(args)
  else:
    raise ValueError('unknown command', args.command)

if __name__ == '__main__': main()



import pytest, sqlparse
from automig.lib import sa_harness, wrappers

def args(classic=True):
  argv = ['schema.sql']
  if classic:
    argv.append('--classic')
  return sa_harness.PARSER.parse_args(argv)

def transform_string(args, string):
  "helper"
  return sa_harness.transform(args, map(wrappers.wrap, sqlparse.parse(string)))

def test_read_globs():
  argv = args()
  "this one is just exercising"
  stmts = sa_harness.read_glob_stmts('test/schema/*.sql')

CASES = {
  'simple': 'create table t1 (a int);',
  'pkey': 'create table t1 (a int primary key, b int);',
  'default': 'create table t1 (a timestamp default now());',
  'index': 'create index i1 on t1(a);',
  'composite': 'create table t1 (a int, b int, primary key (a, b));',
  'enum': "create type letters as enum ('a', 'b');\ncreate table t (col letters);",
}

def test_transform_simple_classic():
  argv = args()
  assert transform_string(argv, CASES['simple']) == \
    ["t1_table = sa.Table('t1', META,\n  sa.Column('a', sa.Integer),\n)"]
  # primary key
  assert transform_string(argv, CASES['pkey']) == \
    ["t1_table = sa.Table('t1', META,\n  sa.Column('a', sa.Integer, primary_key=True),\n  sa.Column('b', sa.Integer),\n)"]
  # default
  assert transform_string(argv, CASES['default']) == \
    ["t1_table = sa.Table('t1', META,\n  sa.Column('a', sa.DateTime, server_default=sa.text('now()')),\n)"]

def test_transform_simple_decl():
  argv = args(False)
  assert transform_string(argv, CASES['simple']) == \
    ["class T1(Base):\n  __tablename__ = 't1'\n  a = sa.Column(sa.Integer)"]
  # primary key
  assert transform_string(argv, CASES['pkey']) == \
    ["class T1(Base):\n  __tablename__ = 't1'\n  a = sa.Column(sa.Integer, primary_key=True)\n  b = sa.Column(sa.Integer)"]
  # default
  assert transform_string(argv, CASES['default']) == \
    ["class T1(Base):\n  __tablename__ = 't1'\n  a = sa.Column(sa.DateTime, server_default=sa.text('now()'))"]

def test_transform_index():
  argv = args()
  assert transform_string(argv, CASES['index']) == \
    ["i1_index = sa.Index('i1', t1_table.c.a)"]

def test_transform_composite_key_classic():
  argv = args()
  assert transform_string(argv, CASES['composite']) == \
    ["t1_table = sa.Table('t1', META,\n  sa.Column('a', sa.Integer, primary_key=True),\n  sa.Column('b', sa.Integer, primary_key=True),\n)"]

def test_transform_composite_key_decl():
  argv = args(False)
  assert transform_string(argv, CASES['composite']) == \
    ["class T1(Base):\n  __tablename__ = 't1'\n  a = sa.Column(sa.Integer, primary_key=True)\n  b = sa.Column(sa.Integer, primary_key=True)"]

def test_transform_enum_classic():
  argv = args()
  assert transform_string(argv, CASES['enum']) == [
    'class lettersEnum(enum.Enum):\n  a = "a"\n  b = "b"',
    "t_table = sa.Table('t', META,\n  sa.Column('col', sa.Enum(lettersEnum)),\n)",
  ]

def test_transform_enum_decl():
  argv = args(False)
  assert transform_string(argv, CASES['enum']) == [
    'class lettersEnum(enum.Enum):\n  a = "a"\n  b = "b"',
    "class T(Base):\n  __tablename__ = 't'\n  col = sa.Column(sa.Enum(lettersEnum))",
  ]



#!/usr/bin/env python3

import os
from distutils.core import setup
from setuptools import find_packages
from automig.version import __version__

setup(
  name='automig',
  version=__version__,
  description="Declarative, automatic db migrations using SQL & git as the source of truth",
  author="Abe Winter",
  author_email="awinter.public+automig@gmail.com",
  url="https://github.com/abe-winter/automigrate",
  packages=find_packages(include=['automig', 'automig.*']),
  entry_points = {
    'console_scripts': [
      'automig=automig.__main__:main',
      'automig_pg=automig.migrate_pg:main',
      'automig_sqlite=automig.automig_sqlite:main',
    ],
  },
  keywords=['sql', 'migration', 'git', 'diff'],
  install_requires=[
    'sqlparse==0.3.0',
    'gitpython==3.1.0',
    'pyyaml>=5.1',
  ],
  extras_require={
    # postgres required for automig_pg script. if install fails, do `apt install libpq-dev`
    "postgres": ["psycopg2-binary==2.8.4"],
  },
  python_requires='>=3.6', # for format strings
  long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
  long_description_content_type='text/markdown',
)
